self.__precacheManifest = (self.__precacheManifest || []).concat([
  {
    "revision": "a2b314eec27db8faaa936f418ad5e8fa",
    "url": "/index.html"
  },
  {
    "revision": "c4b06f652e841a06da91",
    "url": "/static/css/main.95ce86eb.chunk.css"
  },
  {
    "revision": "6b908355fa8223e42727",
    "url": "/static/js/2.368d16db.chunk.js"
  },
  {
    "revision": "9b23de459237d0db01aa743fb7a83d51",
    "url": "/static/js/2.368d16db.chunk.js.LICENSE.txt"
  },
  {
    "revision": "c4b06f652e841a06da91",
    "url": "/static/js/main.1cb23814.chunk.js"
  },
  {
    "revision": "f1fecff66cd973c26d3a",
    "url": "/static/js/runtime-main.7180217b.js"
  }
]);